%------- CALCULATE MEAN DATA FROM RAW DATA (meandata_create.m) --------------------%
% 
%   Creates a new matrix 'meandata' from a source MxN:

%           1       .       .       .       N
%        ------- ------- ------- ------- -------
%     1 |   A   |   B   |   C   |   D   |   E   |
%     .  ------- ------- ------- ------- -------
%       |  a_1  |  b_1  |  c_1  |   .   |   .   |
%     . |  a_2  |  b_2  |   .   |   .   |   .   |
%       |   .   |   .   |   .   |   .   |   .   |
%     . |   .   |   .   |   .   |   .   |   .   |
%       |   .   |   .   |   .   |   .   |   .   |
%     M |  a_M  |  b_M  |       |   .   |   .   |
%        ------- ------- ------- ------- -------

%   Where A-E indicates different nozzles (headers), and (a,b,c,..)_m is 
%   measurements at
%   different constant rotational speeds.
%
%   OUTPUT      meandata        Nx1 struct table
%               meandata_spl    Nx1 struct table
%               nan_map         (M-1)xN table of '0' and '1' where '1' indicates
%                               that the source containes NaN values


%   source      rawdata matrix (from rawdata_import.m)

function [meandata meandata_spl nan_map] = meandata_create(source)


    %----------------------------------------------------------------

    D = 0.4786;             %Diameter of runner [m]
    g = 9.82146514;         %Gravity in the NTNU laboratory
    p_error = 0.207;        %Pressure transducer correction [m] 
    t = 1.960;              %Degrees of freedom (random uncertainty)
    
    n11_a = 36;             %Reduced rot. speed (n_11) start
    n11_b = 44;             %n_11 end
    n11_step = 0.5;         %n_11 increment

    %----------------------------------------------------------------
    
    
    s = size(source);
    nan_map = zeros(s(1),s(2));

    %Create 'meandata'
    
    for j = 1:s(2)-1

        for i = 1:s(1)-1

            if isempty(source{i+1,j})

                p_temp(i,1) = NaN;
                q_temp(i,1) = NaN;
                T_temp(i,1) = NaN;
                M_temp(i,1) = NaN;
                %Mlm_temp(i,1) = NaN;
                %Mtot_temp(i,1) = NaN;
                n_temp(i,1) = NaN;
                q11_temp(i,1) = NaN;
                qed_temp(i,1) = NaN;
                n11_temp(i,1) = NaN;
                ned_temp(i,1) = NaN;
                Ph_temp(i,1) = NaN;
                Pm_temp(i,1) = NaN;
                rho_temp(i,1) = NaN;
                etah_temp(i,1) = NaN;
                head_temp(i,1) = NaN;
                Eh_temp(i,1) = NaN;

                nan_map(i,j) = 1;

            else


                p = source{i+1,j}(:,1);
                q = source{i+1,j}(:,2);
                T = source{i+1,j}(:,3);     %temperature
                M = source{i+1,j}(:,4);     %torque
                %Mlm = source{i+1,j}(:,6);  %torque friction
                n = source{i+1,j}(:,5);

                %Mean values of raw data
                p_temp(i,1) = mean(p);
                q_temp(i,1) = mean(q);
                T_temp(i,1) = mean(T);
                M_temp(i,1) = mean(M);
                %Mlm_temp(i,1) = mean(Mlm);
                %Mtot_temp(i,1) = mean(M) + mean(Mlm);  %total torque
                n_temp(i,1) = mean(n);

                %Calculate error and standard deviation of raw data
                std_p = std(p);
                std_q = std(q);
                std_T = std(T);
                std_M = std(M);
                %std_Mlm = std(Mlm);
                std_n = std(n);

                err_p = (t*std_p)/sqrt(length(p));
                err_q = (t*std_q)/sqrt(length(q));
                err_T = (t*std_T)/sqrt(length(T));
                err_M = (t*std_M)/sqrt(length(M));
                %err_Mlm = (t*std_Mlm)/sqrt(length(Mlm));
                err_n = (t*std_n)/sqrt(length(n)/1000);

                p_temp(i,2) = err_p;
                q_temp(i,2) = err_q;
                T_temp(i,2) = err_T;
                M_temp(i,2) = err_M;
                %Mlm_temp(i,2) = err_Mlm;
                %Mtot_temp(i,2) = sqrt(err_M^2 + err_Mlm^2);
                n_temp(i,2) = err_n;

                p_temp(i,3) = err_p/p_temp(i,1)*100;
                q_temp(i,3) = err_q/q_temp(i,1)*100;
                T_temp(i,3) = err_T/T_temp(i,1)*100;
                M_temp(i,3) = err_M/M_temp(i,1)*100;
                %Mlm_temp(i,3) = err_Mlm/Mlm_temp(i,1);
                %Mtot_temp(i,3) = Mtot_temp(i,2)/Mtot_temp(i,1);
                n_temp(i,3) = err_n/n_temp(i,1)*100;

                etah_temp(i,2) = sqrt(p_temp(i,3)^2 + q_temp(i,3)^2 + ... 
                    T_temp(i,3)^2 + M_temp(i,3)^2 + n_temp(i,3)^2);
                
                %etah_temp(i,2) = sqrt(p_temp(i,3)^2 + q_temp(i,3)^2 + ... 
                    %T_temp(i,3)^2 + Mtot_temp(i,3)^2 + n_temp(i,3)^2);

                %Calc density (rhow), E, H, omega, q11_temp, Ph, Pm, eta, head_temp
                Pstat = p_temp(i,1); 
                rhow = 1000/ ( (1 - (4.6699e-10)*Pstat*1000) + ...
                    (8e-6)*(T_temp(i,1) - 4 + (2.1318913e-7)*Pstat*1000)^2 - ...
                    (6e-8)*(T_temp(i,1) - 4 + (2.1318913e-7)*Pstat*1000)^3 );

                E = ((Pstat*1000)/rhow) + g*p_error + 0.5*(q_temp(i,1)...
                        /(0.25*pi*0.1^2))^2;        
                H = E/g;
                omega = ((2*pi)/60)*n_temp(i,1);  
                q11 = q_temp(i,1)/((D^2)*sqrt(H));
                n11 = (n_temp(i,1)*D)/sqrt(H);
                Ph = rhow*q_temp(i,1)*E;
                Pm = M_temp(i,1)*omega;

                q11_temp(i,1) = q11;
                qed_temp(i,1) = q11/sqrt(g);
                n11_temp(i,1) = n11;
                ned_temp(i,1) = n11/sqrt(g);
                Ph_temp(i,1) = Ph;
                Pm_temp(i,1) = Pm;
                rho_temp(i,1) = rhow;
                etah_temp(i,1) = Pm/Ph;
                head_temp(i,1) = H;
                Eh_temp(i,1) = E;

            end

        end

        %Insert calculated data into new table 'meandata'

        noz = source{1,j};
        b = length(noz)-7;   
        %From raw
        meandata{j,1}.nozzle = noz; %noz(1:b)
        meandata{j,1}.p = p_temp;
        meandata{j,1}.q = q_temp;
        meandata{j,1}.temp = T_temp;
        meandata{j,1}.torque = M_temp;
        %meandata{j,1}.torque_lm = Mlm_temp;
        %meandata{j,1}.torque_tot = Mtot_temp;
        meandata{j,1}.n = n_temp;
        %Calculated
        meandata{j,1}.q11 = q11_temp;
        meandata{j,1}.qed = qed_temp;
        meandata{j,1}.n11 = n11_temp;
        meandata{j,1}.ned = ned_temp;
        meandata{j,1}.power_h = Ph_temp;
        meandata{j,1}.power_m = Pm_temp;
        meandata{j,1}.density = rho_temp;
        meandata{j,1}.etah = etah_temp;
        meandata{j,1}.head = head_temp;
        meandata{j,1}.energy = Eh_temp;
        

        clear q11_temp qed_temp n11_temp ned_temp Ph_temp Pm_temp rho_temp ...
            etah_temp head_temp Eh_temp p_temp q_temp T_temp n_temp
    end

    %   Create 'meandata_spl' where the etah is estimated by a spline and the 
    %   volume flows are avaraged 

    meandata_spl = meandata;

    s = size(meandata_spl);
    
    ideal_ned = (n11_a:n11_step:n11_b)/sqrt(g);
    
    for i = 1:s(1)

        old_eta = meandata{i,1}.etah(:,1);
        old_ned = meandata{i,1}.ned(:,1);
        ss = size(old_eta);

        for ii = 1:ss(1)

            if nan_map(ii,i) == 1   %Datarange contains NaN values >> Must estimate 
                                    %eta and q

                %Calculate average volume flows
                q_sum = 0; q11_sum = 0; qed_sum = 0; mc = 0;
                for m = 1:ss(1)

                    if isnan(meandata{i,1}.q(m,1)) == 0

                        q_sum = q_sum + meandata{i,1}.q(m,1);
                        q11_sum = q11_sum + meandata{i,1}.q11(m,1);
                        qed_sum = qed_sum + meandata{i,1}.qed(m,1);
                        mc = mc + 1;
                    end
                end

                meandata_spl{i,1}.q(ii,1) = q_sum/mc;
                meandata_spl{i,1}.q11(ii,1) = q11_sum/mc;
                meandata_spl{i,1}.qed(ii,1) = qed_sum/mc;

                %Estimate the missing datapoint
                aa = old_ned(1); bb = old_ned(end);
                
                ned_step = (bb - aa)/(length(old_ned)-1);
                
                ned_r = old_ned(1):ned_step:old_ned(end);

                new_eta = spline(old_ned,old_eta(:,1),ned_r);
                
                meandata_spl{i,1}.etah(ii,1) = new_eta(ii);
                
                %n_ED
                meandata_spl{i,1}.ned(ii,1) = ideal_ned(ii); 

            end

        end
    
    end
